/*
 * BaseApplication.java
 *
 * Created on 19 April 2005, 21:03
 */

package jim.common;

import java.sql.*;
import java.util.*;
import javax.mail.Session;
import javax.naming.*;
import javax.sql.DataSource;
import jim.util.NamingUtils;
import jim.util.SqlUtils;


/**
 * The base class for all model-specific code
 * @author briggsj
 */
abstract public class BaseApplication {
    
    protected static BaseApplication Instance = null;
    
    /**
     * Creates a new instance of BaseApplication
     */
    protected BaseApplication() {
    }
    
    /**
     * Return the name of the resource bundle associated with this application's configuration
     * @return String
     */
    abstract public String getBundleName();
    
    private ResourceBundle bundle = null;
    
    /**
     * Get the resource bundle associated with this application's configuration
     * @return ResourceBundle
     */
    protected final ResourceBundle getBundle() {
        if (bundle == null) {
            bundle = ResourceBundle.getBundle(getBundleName());
        }
        return bundle;
    }
    
    /**
     * Get the named property from the application's resource bundle
     * @param name The name of the property
     * @return String
     */
    public final String getProperty(String name) {
        return getBundle().getString(name);
    }
    
    /**
     * Get the specified property from the application's resource bundle.
     * If the property is missing, return the specified default value.
     * @param name name of the property
     * @param dflt default value to return if property not found
     * @return String
     */
    public final String getProperty(String name, String dflt) {
        String value = getBundle().getString(name);
        if (value == null) {
            return dflt;
        } else {
            return value;
        }
    }
    
    private DataSource dataSource = null;
    private String dataSourceName = null;
    
    /**
     * Return the name of the application's database key
     * @return String
     */
    public String getDbName() {
        return getProperty("database");
    }
    
    private String getDataSourceName() {
        return dataSourceName;
    }
    
    /**
     * Return a database connection
     * Create a database connection by finding the first listed datasource called "{dbname}{n}" that can create a valid connection.
     * Once established, keep the successful datasource for subsequent use
     * @throws java.sql.SQLException if no connection can be made
     * @return Connection
     */
    public Connection getConnection() throws SQLException {
        boolean done = false;
        Connection conn = null;
        List<String> attempts = new ArrayList<String>();
        
        if (dataSource == null) {
            String name = this.getDbName();
            
            int num = 0;
            Context envCtx;
            try {
                envCtx = NamingUtils.findContext();
            } catch (NamingException e) {
                throw new SQLException("Cannot initialise context");
            }
            do {
                String key = name + num;
                try {
                    DataSource ds = (DataSource)envCtx.lookup(key);
                    if (ds != null) {
                        try {
                            attempts.add(key);
                            conn = ds.getConnection();
                            dataSource = ds;
                            dataSourceName = key;
                            Trace.print("Established connection to database " + key);
                            done = true;
                        } catch (SQLException e) {
                            Trace.print("Could not open " + key + ": " + e.getMessage());
                        }
                    }
                } catch (NamingException e) {
                    Trace.print("Could not open " + key + ": " + e.getMessage());
                }
                
                num++;
            } while (!done && (num < 10));
            if (conn == null) {
                throw new SQLException("No database can be opened - tried: " + attempts.toString());
            }
        } else {
            Trace.print("Established connection using existing datasource: " + dataSourceName);
            conn = dataSource.getConnection();
        }
        return conn;
    }

    private SqlUtils sql = null;
    
    public SqlUtils getSqlUtils () throws SQLException {
        if (sql == null) {
            Connection conn = this.getConnection();
            sql = new SqlUtils(conn);
        }
        return sql;
    }
    
    public void closeSqlUtils() throws SQLException {
        sql.getConn().close();
        sql = null;
    }
    
    /**
     * Getter for property version.
     * @return Value of property version.
     */
    public String getVersion() {
        return getBundle().getString("version");
    }
    
    /**
     * Get the singleton instance of this class
     * @return {@link BaseApplication}
     */
    public static BaseApplication getInstance() {
        return null;
    }
    
    private Session session = null;
    private boolean testing = false;
    
    /**
     * Return a JavaMail session associated with this application
     * @return <CODE>javax.mail.Session</CODE>
     */
    public Session getEmailSession() {
        if (session == null) {
            Properties props = System.getProperties();
            String host = getProperty("mailhost", "smtpserver.port.ac.uk");
            props.put("mail.smtp.host", host);
            session = Session.getDefaultInstance(props, null);
            
            String test = getProperty("mailtest", "test");
            if ("test".equalsIgnoreCase(test)) {
                testing = true;
            }
        }
        return session;
    }

    /**
     * Return the dataSource associated with the database connection
     * @return DataSource
     */
    public DataSource getDataSource() {
        return dataSource;
    }
}
