/*
 * GenericMessageFetcher.java
 *
 * Created on 19 April 2005, 12:02
 */

package jim.common.mail;

import java.util.*;
import jim.common.StringFormatter;

/**
 * Generic class for fetching email messages from various sources
 * @author  briggsj
 */
public abstract class GenericMessageFetcher {
    
    /**
     * Holds all message components.
     */
    Map<String, String> messages;
    
    /** Creates a new instance of GenericMessageFetcher */
    public GenericMessageFetcher() {
        messages = new HashMap<String,String>();
    }
    
    /**
     * Fetch an email message with the specified name
     * @param messageName The name of the message to be fetched.
     * @return {@link EmailMessage}
     */
    public EmailMessage fetch(String messageName) {
        return fetch(messageName, false);
    }
    
    /**
     * Fetch an email message with the specified name. If force is true, it will be
     * fetched from storage even if already read.
     * @param messageName Name of message to be fetched.
     * @param force If true, the message will be fetched from storage even if already read.
     * @return {@link EmailMessage}
     */
    public EmailMessage fetch(String messageName, boolean force) {
        EmailMessage msg = new EmailMessage();
        
        load(messageName, force, null);
        
        if (! messageName.endsWith("_subject")) {
            String subjectKey = messageName + "_subject";
            load(subjectKey, force, "No subject");
            String subject = getString(subjectKey);
            msg.setSubject(subject);
        } else {
            msg.setSubject("");
        }
        msg.setGeneralParams(messages);
        String text = getString(messageName);
        msg.setText(text);
        return msg;
    }
    
    /**
     * Load a set of messages and components.
     * @param rootName The name of the root message (component).
     * @param force Force the component to be read.
     * @param dflt Value to be used if rootName can't be found.
     * @throws MessageException if the message component does not exist and <CODE>dflt</CODE> is null
     */
    protected void load(String rootName, boolean force, String dflt) throws MessageException {
        if (force || getString(rootName) == null) {
            String text = null;
            Exception ex = null;
            try {
                text = fetchString(rootName);
            } catch (MessageException e) {
                text = null;
                ex = e;
            }
            if (text == null) {
                if (dflt == null) {
                    throw new MessageException("No value for message " + rootName, ex);
                } else {
                    text = dflt;
                }
            }
            setString(rootName, text);
            
            //Iterate through all {substrings}
            List fields = StringFormatter.findComponents(text);
            Iterator it = fields.iterator();
            while (it.hasNext()) {
                String key = (String)it.next();
                //recurse, loading all strings mentioned in this string's text
                load(key, force, "[" + key + " not a message component]");
            }
        }
    }
    
    /**
     * Extract a string from the messsages pool.
     * @param stringName The name of the component to be returned.
     * @return String
     * @throws MessageException If a message or component cannot be found.
     */
    public abstract String fetchString(String stringName) throws MessageException;
    
    /**
     * Load all strings from the database
     * @throws MessageException if there are database problems
     */
    public abstract void fetchAllStrings() throws MessageException;
    
    /**
     * Store (update/insert) the specified message into the pool
     * @param stringName the name of the component to be stored
     * @param value the value of the component to be stored
     */
    public abstract void storeString(String stringName, String value) throws MessageException;
    
    /**
     * Set the value of a component in the message pool.
     * @param stringName The name of the component.
     * @param value The value of the component.
     */
    public void setString(String stringName, String value) {
        messages.put(stringName, value);
    }
    
    /**
     * Get the value of a message component.
     * @param stringName The name of the component.
     * @return String
     */
    public String getString(String stringName) {
        return (String)messages.get(stringName);
    }
    
    /**
     * Obtain set of message keys
     * @return list of {@link String} containing names of keys
     */
    public Set keySet() {
        return messages.keySet();
    }
    
    /**
     * Get all message components
     * @return a map containing each message component keyed by its name
     */
    public Map<String,String> getMessages() {
        return messages;
    }
}
